#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/wait.h>
#include <sys/param.h>
#include <sys/types.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/wmgeneral.h"
#include "../wmgeneral/misc.h"

#include <dirent.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>

void init_stuff(int argc, char **argv);
int main_loop();

extern XpmIcon wmgen;
extern GC NormalGC;
void poner_marco();
void default_image();

void get_next_background();
void get_prev_background();
int myselect(const struct dirent *d);
void load_background(char *name);
void myexecCommand(char *command);
void show_usage();
void add_background(char *filename);
